module Mondrian

    class TokenEncoder
        TokenCharacters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_' unless defined? TokenCharacters
        TokenFillCharacter = '~' unless defined? TokenFillCharacter
        TokenNullCharacter = '.' unless defined? TokenNullCharacter
                
        def initialize(symbols)
            symbols.each { |symbol|
                add_token symbol
            }
        end
        
        
        def match(symbol)
            return match_any unless symbol
            
            /#{@tokens[symbol]}#{TokenFillCharacter}*/
        end


        def match_any
            /\w#{TokenFillCharacter}*/o
        end
        
        
        def token_symbol(encoded_token)
            @tokens.invert[encoded_token[0].chr]
        end


        def encode_token(symbol, length)
            @tokens[symbol] + TokenFillCharacter * (length - 1)
        end
        
        
        def encoded_token_is?(encoded_token, symbol)
            token_symbol(encoded_token) == symbol
        end


        def count_tokens(encoded_tokens)
            encoded_tokens.count(TokenCharacters)
        end
        
        
        def null_pad(length)
            TokenNullCharacter * length
        end
        
        
        protected
        def add_token(symbol)
            @tokens = {} unless defined? @tokens
        
            @tokens[symbol] = TokenCharacters[@tokens.size].chr
                
            add_token_methods(symbol)
        end
        
        
        protected
        def add_token_methods(symbol)
            eval <<-fin
                def #{symbol}
                    /#{@tokens[symbol]}#{TokenFillCharacter}*/o
                end
                
                
                def not_#{symbol}
                    /[^#{@tokens[symbol]}#{TokenFillCharacter}#{TokenNullCharacter}]#{TokenFillCharacter}*/o
                end
            fin
        end

    end

end    


