# A storage place for useful but nontrivial methods.
#
# Contributors:: Oliver Smith
#

module Mondrian

    # Converts a string to a simple regexp which matches that string.

    def regexp_from_string(string)
        /^#{Regexp.escape(string)}$/
    end


    # Returns +true+ if +path+ is a relative path.
    
    def is_relative?(path)
        if platform_is_windows?
            return path[0].chr == '.' || (path[0].chr != File::SEPARATOR && path[0].chr != File::ALT_SEPARATOR && path[1].chr != ':')
        else
            return path[0].chr == '.' || (path[0].chr != File::SEPARATOR && path[0].chr != File::ALT_SEPARATOR)
        end
    end


    # Returns +true+ if +path+ is an absolute path.

    def is_absolute?(path)
        not is_relative? path
    end


    # Returns +true+ if +path+ is an actual directory and not a link.

    def is_real_directory?(path)
        return File.directory?(path) && (not File.basename(path) =~ /\A\.\.?\Z/) && (not File.symlink?(path))
    end
    
    
    # Returns +true+ if +subpath+ is a subpath of +path+.
    
    def is_subpath?(subpath, path)
        if platform_is_windows?
            return File.expand_path(subpath).downcase.index(File.expand_path(path).downcase) == 0
        else
            return File.expand_path(subpath).index(File.expand_path(path)) == 0
        end
    end
    
    
    def platform_is_windows?
        return PLATFORM =~ /mswin/
    end

end