module Mondrian

    module LexerInternals

        def self.append_features(receiver)
            super
            
            receiver.extend(ClassMethods)
        end
    
        
        def tokens
            self.class.tokens
        end
        
        
        module ClassMethods
        
            def tokens
                (defined? @tokens) ? @tokens : []
            end
            
        
            private
            def token(*symbols)
                symbols.each { |symbol|
                    add_token(symbol)
                }
            end
        
        
            private
            def add_token(symbol)
                @tokens = [] unless defined? @tokens
            
                @tokens << symbol
            end

        end    

    end



    class Lexer
        include LexerInternals            
    
    
        def keyword_regexp(keywords)
            Regexp.new('\A(' + keywords.map { |keyword| Regexp.escape(keyword) }.join('|') + ')\b')
        end


        def operator_regexp(operators)
            Regexp.new('\A(' + operators.map { |operator| Regexp.escape(operator) }.join('|') + ')')
        end
        
        
        def next_token(code, context)
            raise 'Base class method called!'
        end
    end
    
    
    
    class LexerContext
        attr_accessor :last, :last_token
        
        def initialize(last, last_token)
            @last = last
            @last_token = last_token
        end
        
    
        def update!(last, last_token)
            @last = last
            @last_token = last_token
            
            false
        end

    end

end
