require 'framework/mixins/editable'

module Mondrian

    class TextEditor
        include Editable
        
        event :text_inserted, :text_deleted, :text_lexed
        
        attr_reader :tokenstring
        
        def initialize(tokenstring)
            @tokenstring = tokenstring
        end
        
        
        def insert_text(text, position)
            affected_range = @tokenstring.insert_text(text, position)
            
            notify_text_inserted text, position
            notify_text_lexed affected_range
        end


        def delete_text(text, position)
            affected_range = @tokenstring.delete_text(text, position)
            
            notify_text_deleted text, position
            notify_text_lexed affected_range
        end


        def method_missing(method_symbol, *args, &block)
            @tokenstring.send(method_symbol, *args, &block) 
        end

    end
    
end